#!/usr/bin/env ruby
require_relative 'config'
require './lib/cli_option_parser'
require './lib/tester'
require 'fileutils'

$options = CLIOptionParser.parse(ARGV)
FileUtils.rm_rf($TMP_DIR) if File.exists?($TMP_DIR)
FileUtils.mkdir $TMP_DIR
FileUtils.mkdir $LOG_DIR unless File.exists?($LOG_DIR)
$recursive = $options.recursive
$verbose = $options.verbose
$tester = Tester.new($options)

$tester.add(EfficiencyTest.new) if $options.test.include? 'all' or $options.test.include? 'efficiency'
if $options.test.include? 'all' or $options.test.include? 'single_common_block'
  $tester.add(SingleCommonBlockTest.new({iterations: 5, filesize_factor: 512, reduction_factor: 16, step_size: 5, overwrite: true}))
  $tester.add(SingleCommonBlockTest.new({iterations: 5, filesize_factor: 2048, reduction_factor: 16, step_size: 5, overwrite: true}))
  $tester.add(SingleCommonBlockTest.new({iterations: 5, filesize_factor: 8192, reduction_factor: 16, step_size: 5, overwrite: true}))
end
if $options.test.include? 'all' or $options.test.include? 'fragment'
  $tester.add(FragmentDetectionTest.new({cut_end_only: false, step_size: 5}))
  $tester.add(FragmentDetectionTest.new({cut_end_only: true, step_size: 5}))
end
if $options.test.include? 'all' or $options.test.include? 'alignment'
  $tester.add(AlignmentTest.new({step_size: 10, max_size: 100, fixed_size: false}))
  $tester.add(AlignmentTest.new({step_size: 100, max_size: 500, fixed_size: false}))
  $tester.add(AlignmentTest.new({step_size: 1024, max_size: 4*1024, fixed_size: true}))
  $tester.add(AlignmentTest.new({step_size: 4096, max_size: 64*1024, fixed_size: true}))
end
$tester.add(ObfuscationResistanceTest.new({step_size: 10})) if $options.test.include? 'all' or $options.test.include? 'obfuscation'

$tester.full_test