require 'optparse'
require 'ostruct'

# This class takes care of the command line options.
#
# Details can be viewed with the +-h+ or +--help+ flag in the command line.
class CLIOptionParser
  
  def self.parse(args)
    options = OpenStruct.new

    parser = OptionParser.new do |opts|
      opts.banner = "Usage: sphtest [options] PATH"
    
      options.test = ['all']
      opts.on('-t', '--test [TESTS]', String, 'Select tests to be performed. Possible options are efficiency, single_common_block, fragment, alignment and obfuscation. By default all tests are performed.') do |tests|
        options.test = tests.split(',')
      end
      
      options.verbose = false
      opts.on('-v', '--verbose', 'Display statistics for every file, as it is processed (efficiency test only).') do
        options.verbose = true
      end
      
      options.recursive = false
      options.max_depth = 1
      opts.on('-r', '--recursive [MAX_DEPTH]', Integer, 'Recurse into folders.') do |max_depth|
        options.recursive = true
        options.max_depth = max_depth
      end
      
      opts.on( '-h', '--help', 'Display this help text.' ) do
        puts opts
        exit
      end
    end
      
    begin 
      raise OptionParser::InvalidOption if args.empty?
      parser.parse!(args)
    rescue OptionParser::InvalidOption => e
      puts parser
      exit 1
    end
    options
  end
  
end