require_relative 'fuzzy_hash_function'

# This class inherits from BaseHashFunction.
#
# It sets the \sdhash program that will be used and computes the digest length after parsing the output of the program.
class SDHASH < FuzzyHashFunction
  
  PROGRAM_NAME = "sdhash"

  def initialize(params="")
    super(params)
    @all_pair_comparison_command = PROGRAM_NAME + " -c"
    @gen_compare_command = PROGRAM_NAME + " -t -1 -g"
  end

  # Example digest:
  #
  # sdbf:03:23:/var/fileset/0000_5.odt:3711:sha1:256:5:7ff:160:1:63:hWIAAKQIAissBgBJEIABAAIgB........AAEkoAEAASAAAA==
  def self.file_stats(digest)
      if digest == ""
        info_hash = { filename: "", filesize: 0, digest: "", digest_length: 0}
      else
        info_hash = {}
        info_hash[:filename] = digest.split(':')[3] # "/var/fileset/0000_5.odt"
        l = digest.split(':'); l.delete_at(2); l.delete_at(2); # ["sdbf", "03", "3711", "sha1", "256", "5", "7ff", "160", "1", "63", "hWIAAKQIAissBgBJEIABAAIgB........AAEkoAEAASAAAA=="]
        info_hash[:filesize] = File.size info_hash[:filename]
        info_hash[:digest] =  l.join(':') # sdbf:03:3711:sha1:256:5:7ff:160:1:63:hWIAAKQIAissBgBJEIABAAIgB........AAEkoAEAASAAAA==
        info_hash[:digest_length] = self.digest_length(info_hash[:digest])
      end
      return info_hash 
  end
  
  # The digest consists of several parts, divided by a colon. All parts except the last one count as 1 byte per character.
  #
  # The last part is base64-encoded, so its length is multiplied with 3/4.
  def self.digest_length(digest)
    return 0 if digest.nil? or digest.empty?
    base64_part = digest.split(':')[-1]
    digest.length + (base64_part.length * 3 / 4) - base64_part.length
  end
  
  # Example match:
  #
  # /var/fileset/0000_10.odt|/var/fileset/0000_15.odt|100
  def self.split_match(match)
      parts = match.split('|') # parts = ["/var/fileset/0000_10.odt", "/var/fileset/0000_15.odt", "100"]
      left = File.basename(parts[0]).split('_')[0] # left = "0000"
      right = File.basename(parts[1]).split('_')[0] # right = "0000"
      score = parts[2].to_i # score = 100
      [left, right, score]
  end
end