require 'securerandom'

class FileManipulator
  attr_accessor :mod_log
  
  def initialize
    reset
  end
  
  def reset
    @mod_log = {substitute: 0, delete: 0, insert: 0}
  end
  
  def self.set_pos(file=nil, pos=nil)
    filesize = File.size(file)
    return 0 if filesize == 0
    return generate_random_number filesize-1 if pos.nil?
    return filesize if pos >= filesize
    pos
  end
    
  def self.set_side(side=nil)
    return generate_random_number 1 if side.nil?
    return side if side == 0 or side == 1
    return 1 if side > 1
    return 0 if side < 0
  end
  
  def manipulate_file!(file=nil, percent=nil, op_index=nil)
    operations = ["substitute", "delete", "insert"]
    count = percent.nil? ? 1 : percent_in_bytes(file, percent)
    print("count = ", count) if $debug
    count.times do
      begin
        if op_index.nil? or op_index.to_i < 0 or op_index.to_i >= operations.size
          op_index = FileManipulator.generate_random_number operations.size-1
        end
        method = operations[op_index]+"_byte!"
      end while not FileManipulator.send(method, file)
      @mod_log[operations[op_index].to_sym] += 1
      pp @mod_log if $debug
    end
  end

  def self.add_chunk!(file=nil, chunk_size=nil, pos=nil, chunk=nil, overwrite=false)
    exit 1 if file.nil?
    if chunk_size.nil? ^ chunk.nil?
      chunk_size ||= 1
      chunk ||= generate_random_bytes chunk_size
      pos = set_pos(file, pos)
      content = IO.binread(file)
      overwrite ? content[pos...(pos+chunk.length)] = chunk : content = content[0...pos].to_s + chunk + content[pos..-1].to_s
      IO.binwrite(file, content)
    else
      puts "\nSize and content given. Please select one of those."      
    end
    [content, pos]
  end

  def self.cut_chunk!(file=nil, number_of_bytes=0, side=nil)
    exit 1 if file.nil?
    side = set_side(side)
    content = IO.binread(file)
    content = side == 0 ? content[number_of_bytes..-1] : content[0...-number_of_bytes]
    IO.binwrite(file, content)
    [content, side]
  end
      
  def self.delete_byte!(file=nil, pos=nil)    
    exit 1 if file.nil?
    pos = set_pos(file, pos)
    content = IO.binread(file)
    return false if content.empty?
    byte = content[pos]
    content = content[0...pos] + content[(pos+1)..-1]
    IO.binwrite(file, content)
    [byte, pos]
  end
  
  def self.insert_byte!(file=nil, byte=nil, pos=nil)
    exit 1 if file.nil?
    byte ||= generate_random_bytes 1
    pos = set_pos(file, pos)
    content = IO.binread(file)
    content = content[0...pos].to_s + byte.chr + content[pos..-1].to_s
    IO.binwrite(file, content)
    [byte, pos]
  end
  
  def self.substitute_byte!(file=nil, byte=nil, pos=nil)
    exit 1 if file.nil?
    byte ||= generate_random_bytes 1
    pos = set_pos(file, pos)
    IO.binwrite(file, byte.chr, pos)
    [byte, pos]
  end
  
  def self.generate_random_bytes(size=1)
    SecureRandom.random_bytes size
  end
  
  def self.generate_random_number(lower_bound=0, upper_bound=0)
    if lower_bound > upper_bound
      lower_bound, upper_bound = upper_bound, lower_bound
    end
    max = upper_bound - lower_bound + 1
    SecureRandom.random_number(max) + lower_bound
  end

  def self.generate_random_file(size=nil, dir=nil, filename=nil, content=nil)
    if size.nil? ^ content.nil?
      content ||= SecureRandom.random_bytes(size)
      Dir.chdir(dir) unless dir.nil?
      filename = Time.now.strftime("%Y_%m_%d__%H_%M_%S")+".bin" if filename.nil?
      file = File.new(filename, 'wb')
      file.print content
      file.close
      return filename
    else
      puts "\nSize and content given. Please select one of those."
    end
  end
  
end