require 'action_view'
require 'colored'
require 'terminal-table'
require 'pp'

#TODO remove colored tags for printing to file
module Printer
  
  PRECISION = { low: 2, mid: 3, high: 5}
  DEFAULT_PRECISION = PRECISION[:high]
  
  def current_datetime
      Time.now.strftime("%d.%m.%Y %H:%M:%S")
  end
  
  def precision(number, precision=DEFAULT_PRECISION)
    number.is_a?(Numeric) ? "%.#{precision}f" % number : number
  end
  
  def align_each_with_precision(values, prec=DEFAULT_PRECISION, align=:right)
    values.map{ |v| value = v.nil? ? "-" : v; align_with_precision(value, prec, align) }
  end

  def align_each(values, align=:right)
    values.map{ |v| value = v.nil? ? "-" : v; align(value, align) }
  end

  def align_with_precision(value, prec=DEFAULT_PRECISION, align=:right)
    { value: precision(value, prec), alignment: align }
  end
  
  def align(value, align=:right)
    { value: value, alignment: align }
  end
  
  def as_bold(string)
    $COLORED_OUTPUT ? string.to_s.bold : string
  end
  
  def as_underlined_header(string)
    $COLORED_OUTPUT ? string.to_s.bold.blue.underline : string
  end
  
  def as_header(string)
    $COLORED_OUTPUT ? string.to_s.yellow : string
  end
  
  def as_important_header(string)
    $COLORED_OUTPUT ? string.to_s.bold.blue : string
  end
  
  def as_status(string)
    $COLORED_OUTPUT ? string.to_s.green : string
  end
  
  def as_info(string)
    $COLORED_OUTPUT ? string.to_s.yellow : string
  end

  def as_error(string)
    $COLORED_OUTPUT ? string.to_s.red : string
  end
  
  def split_number(number, chars=3)
    number.to_s.reverse.gsub(/...(?=.)/,'\& ').reverse
  end
  
  def to_human(size)
    output = size < 1024 ? "#{split_number(size)} B" : "#{$size_converter.number_to_human_size(size)} (#{split_number(size)} B)"
  end
      
end