class BaseLogAnalyzer
  
  attr_accessor :stats
  attr_accessor :file
  attr_accessor :matches
  attr_accessor :results

  def initialize(params={})
    reset
    @stats = []
  end
  
  def reset
    @file = { name: nil, values: {} }
    @matches = []
    @results = []
  end
  
  def standard_deviation
    variance.map { |v| v.nil? ? v : (v**0.5)}
  end

  def variance
    variances = []
    means.each_with_index do |m, i|
      if m.nil?
        variances << m
      else
       valid_values = @results.transpose[i].compact
       variances << (valid_values.inject(0) { |sum, s| sum + (s-m)**2 }/valid_values.length.to_f)
      end
    end
    variances
  end  
  
  def values
    @stats.map { |file| file[:values].values }
  end
  
  def means
    @results.transpose.map {|x| x.compact!; @matches << (x.length == 0 ? nil : x.length) if @matches.length < @results[0].length; x.reduce(:+)/x.length.to_f if x.any? }.flatten
  end
  
  def fill_results(length, fill_value=nil)
    @results = values.map! { |values_arr| values_arr.fill(fill_value, values_arr.length...length) }
  end
  
  def means_in_percent
    means.map {|mean| 100*mean/$avg_filesize if mean}
  end
  
  def fill_stats(max, step, fill_value=nil)
    @stats.each do |stat|
      tmp = stat[:values]
      stat[:values] = {}
      (max-step).step(1, -step).map { |key| stat[:values].merge!({ key => tmp[key]}) }
      stat[:values][0] = tmp[0]
    end
    @results = values
  end
end