# Automatically load all testers (i.e. all files in lib/tester/ which end with _test.rb)
Dir.glob($SCRIPT_DIR + '/lib/testers/*_test.rb', &method(:require))
require_relative './helpers/helper'
require_relative './helpers/printer'

include Helper
include Printer
$size_converter = ActionView::Base.new
$file_count = 0
$total_filesize = 0
$largest_file = nil
$total_duration = 0.0
$avg_filesize = 0.0

# This class provides the specific test methods (efficiency, security).
#
# Furthermore it includes helper methods to traverse the directories and to split files from folders.
class Tester
  def initialize(options=nil)
    begin
    raise "No options given.\nBe sure to pass options to the Tester object when creating it." if options.nil?
    rescue RuntimeError => e
      puts e
      exit 1
    end
    @options = options
    @tester = nil
    @testers = []
    @results = []
    unless @options.test === ["single_common_block"]
      @dirs = ARGV.select { |f| File.directory?(f) }.sort
      @filenames = (ARGV.select { |f| File.file?(f) }.map{ |p| File.expand_path(File.dirname(p)) + '/' + File.basename(p)}.flatten).sort
      @all_filenames = Array.new(@filenames)
      @all_filenames << find_all_files(@dirs) unless @dirs.empty?
      @all_filenames.flatten!
      exit_if_no_files_given(@filenames, @dirs)
    end
  end
  
  def add(tester)
    @testers << tester if tester.is_a? BaseTest
  end
  
  def full_test
    pwd = Dir.pwd
    Dir.chdir $TMP_DIR
    @testers.length
    $total_duration = realtime { @testers.each do |tester|
      if tester.is_a? EfficiencyTest
        tester.test(@filenames, @dirs, @all_filenames)
      else
        tester.takes_input ? tester.test(@all_filenames) : tester.test
      end
    end
    }
    print
    create_report_file
    Dir.chdir pwd
  end
  
  def compute_total_filesize
    filesize = @all_filenames.inject(0) { |sum, file| sum + File.size(file) }
  end
  
  def find_largest_file
    lf = @all_filenames.max_by { |f| File.size(f) }
    { name: lf, size: File.size(lf)}
  end
  
  def build_results
    Dir.chdir $TMP_DIR
    $file_count = @all_filenames.nil? ? 0 : @all_filenames.length
    if $file_count > 0
      $total_filesize = compute_total_filesize
      $largest_file = find_largest_file
      $avg_filesize = $total_filesize/$file_count.to_f
    end
    if $file_count > 0 or @options.test === ["single_common_block"]
      @results = [summary_table]
      @testers.each { |t| @results << t.results }      
    else
      puts as_error("No files were processed.")
    end    
  end

  def create_report_file
      $COLORED_OUTPUT = false
      File.open("#{$SCRIPT_DIR}/log/report_#{Time.now.strftime("%Y_%m_%d__%H_%M_%S")}.txt", 'w') do |f|
        f.puts $COMMAND, "\n"
        build_results
        f.puts @results.flatten.join("\n\n") if @results.is_a? Array
      end
      $COLORED_OUTPUT = true
  end
  
  def summary_table
    if @options.test === ["single_common_block"]
      rows = [["Total duration", align("~ "+$total_duration.duration)]]
    else
      rows = []
      rows << ["File count", align($file_count)]
      rows << ["Total duration", align("~ "+$total_duration.duration)]
      rows << ["Avg. file size", align_with_precision(to_human(precision($avg_filesize, PRECISION[:low]).to_f))]
      rows << ["Total file size", align(to_human($total_filesize))]
      rows << ["Largest file", align("'#{$largest_file[:name]}' with #{to_human($largest_file[:size])}")]
    end
    table = Terminal::Table.new :title => as_important_header("Summary"), :rows => rows, :style => { border_x: '=', border_i: '.' }
  end

  def print
    build_results
    puts @results.flatten.join("\n\n") if @results.is_a? Array
  end
    
end
