require_relative 'base_test'

class AlignmentTest < BaseTest
  def initialize(params={step_size: 0.5, max_size: 10, fixed_size: false})
    super
    @hash_functions.each { |h| h.log.merge!({ alignment: BaseLogAnalyzer.new }) }
    @fixed_size = params[:fixed_size]
    @step_size = params[:step_size]
    @max_size = params[:max_size]
    @log_key = :alignment
  end
  
  
  
  protected
  
  def compute_chunk_size
    @chunk_size = @fixed_size ? @step_size : percent_in_bytes(@file, @step_size)
  end
  
  def set_method
    @fixed_size ? "fixed size" : "percentual"
  end
  
  def set_headings
    [as_header("Added #{'(%)' unless @fixed_size}")] << @index_iterator.map { |x| align(as_header("#{@fixed_size ? $size_converter.number_to_human_size(x) : x}")) }
  end
  
  def set_title(method)
    step_size = @fixed_size ? $size_converter.number_to_human_size(@step_size) : @step_size.to_s + " %"
    max_size = @fixed_size ? $size_converter.number_to_human_size(@max_size) : @max_size.to_s + " %"
    as_important_header("Alignment test result (#{method+', '}step size = #{step_size}, max size = #{max_size})")
  end
  
  
  
  private
  
  def continue?
    @hash_functions.select{|x| x.is_active? }.any? and @steps_done < @max_size
  end
  
  def process_file
    FileManipulator.add_chunk!(@copy, @chunk_size, 0)
  end
  
end
