require 'securerandom'
require_relative 'base_test'
require_relative '../helpers/file_manipulator'

class FragmentDetectionTest < BaseTest

  def initialize(params={cut_end_only: false, step_size: 5})
    super
    @cut_end_only = params[:cut_end_only]
    @max_size = 100
    @start_side = @cut_end_only ? 1 : nil
    @start_percentage = params[:step_size]
    @step_size = @start_percentage
    @side = @start_side
    @chunk_size = 0
    @hash_functions.each { |h| h.log.merge!({ fragment: BaseLogAnalyzer.new }) }
    @log_key = :fragment
  end
  
  
  
  protected
  
  def reset_parameters
    @side = @cut_end_only ? 1 : FileManipulator.set_side(@start_side)
    @step_size = @start_percentage
    @alternate = !@cut_end_only
  end
  
  def compute_chunk_size
    @chunk_size = percent_in_bytes(@file, @step_size)
  end
  
  def set_method
    @cut_end_only ? "cut side: right (end only)" : "cut side: random start, then alternating"
  end
  
  def set_headings
    [as_header("File size (%)")] << @index_iterator.map { |x| align(as_header("#{@max_size-x}"))}
  end
  
  def set_title(method)
    as_important_header("Fragment detection test result (#{method+', '+@start_percentage.to_s} %)")
  end
  
  
  
  private
  
  def continue?
    if @max_size - @steps_done <= @step_size
      @chunk_size /= @step_size
      @step_size = 1
    end
    return true if @steps_done < @max_size-1 and @hash_functions.select{|x| x.is_active?}.any?
    false
  end
  
  def process_file
    FileManipulator.cut_chunk!(@copy, @chunk_size, @side)[1]
    @side = @alternate ? (@side.even? ? 1 : 0) : (@cut_end_only ? 1 : FileManipulator.set_side)
  end
    
end
