require_relative 'base_test'

class ObfuscationResistanceTest < BaseTest
  def initialize(params={step_size: 10})
    super
    @step_size = params[:step_size]
    @fm = FileManipulator.new
    @log_key = :obfuscation
    @limit = {}
    @hash_functions.each do |h|
      h.log.merge!({ @log_key => BaseLogAnalyzer.new})
      @limit.merge!({h.key => @max_size-@step_size })
    end
    @max_limit, @min_limit = 0, 100
  end

  def results
    super { |log| log.fill_stats(100,@step_size) }
  end
  
  
  
  protected
  
  def reset_parameters
    @fm.reset
    @edit_count = 0
    @hash_functions.each { |h| @limit.merge!({h.key => 100 }) }
  end
  
  
  
  private
  
  def process_file
    @fm.manipulate_file!(@copy)
    @edit_count += 1
  end
  
  def update_file_stats
    @steps_done += @step_size
    @hash_functions.select{|x| x.is_active? }.each do |h|
      h.compute_score(@file, @copy)
      score = h.score < 0 ? 0 : h.score
      @limit[h.key] = (score - score%@step_size) if score < @max_size
      @max_limit = @limit[h.key] if @max_limit < @limit[h.key] and @limit[h.key] < @max_size
      @min_limit = @limit[h.key] if @min_limit > @limit[h.key] and @limit[h.key] >= @min_size
      if score >= @limit[h.key]
        h.log[@log_key].file[:values].merge!({ @limit[h.key] => @edit_count })
        h.previous_score = score
      end
      h.deactivate if h.score <= 0
    end
  end
  
  def teardown
    super
    @index_iterator = @max_limit.step(@min_limit, -@step_size).map { |x| x}
  end

  def set_headings
    [as_header("Score")] << @index_iterator[0...-1].map { |x| align(as_header(">= #{x}"))} << align(as_header(0))
  end
  
  def set_title(method)
    as_important_header("Obfuscation resistance test result")
  end
    
  def set_rows(h, log, h_index)
    @rows << [[as_bold(h.class::PROGRAM_NAME)] << log.means.map { |m| ""}].flatten
    @rows << [["Avg. changes"] << align_each_with_precision(log.means, PRECISION[:low])].flatten
    @rows << [["Avg. changes (%)"] << align_each_with_precision(log.means_in_percent)].flatten      
    @rows << [["Matches"] << align_each(log.matches)].flatten
    @rows << [["Variance"] << align_each_with_precision(log.variance) ].flatten
    @rows << [["Std. deviation"] << align_each_with_precision(log.standard_deviation) ].flatten
    @rows << :separator if (h_index < @hash_functions.length-1)
  end
  
end
