require_relative 'base_test'

class SingleCommonBlockTest < BaseTest

  KB = 1024

  def initialize(params={iterations: 5, filesize_factor: 512, reduction_factor: 16, step_size: 5, overwrite: true})
    @filesize_factor = params[:filesize_factor]
    @reduction_factor = params[:reduction_factor]
    @step_size = params[:step_size]
    @iterations = params[:iterations]
    super
    @log_key = :single_common_block
    @overwrite = params[:overwrite]
    @limit = {}
    @hash_functions.each do |h|
      h.log.merge!({ @log_key => BaseLogAnalyzer.new})
      @limit.merge!({h.key => @max_size-@step_size })
    end
    @max_limit, @min_limit = 0, 100
    @takes_input = false
  end
  
  def test
    @iterations.times { |i| test_single_common_block }
  end
  
  def results
    super { |log| log.fill_stats(@max_limit+@step_size,@step_size)}
  end
  
  
  
  private
  
  def test_single_common_block
    setup
    while continue?
      @files.each do |file|
        @file = file
        process_file
      end
      update_file_stats
      @common_block = @common_block[0...-@reduction_factor*KB]
    end
    teardown
  end
  
  def setup
    @hash_functions.each { |h| h.reset; h.log[@log_key].reset }
    @filesize_factor ||= 512; @reduction_factor ||= 16; @filesize = @filesize_factor*KB; @common_block_size = @filesize/2
    @common_block = FileManipulator.generate_random_bytes @common_block_size
    @common_block
    @files = [{ name: "f1", pos: nil, copy: nil }, { name: "f2", pos: nil, copy: nil }]
    @files.each do |file|
      FileManipulator.generate_random_file(@filesize, $TMP_DIR, file[:name])
      file[:pos] = FileManipulator.generate_random_number(@filesize-@common_block_size)
    end
  end
    
  def process_file
    FileUtils.cp @file[:name], @file[:name]+"_copy"
    @file[:copy] = @file[:name]+"_copy"
    FileManipulator.add_chunk!(@file[:copy], chunk_size=nil, @file[:pos], @common_block, @overwrite)
  end
  
  def update_file_stats
    @hash_functions.select{|x| x.is_active? }.each do |h|
      h.compute_score @files[0][:copy], @files[1][:copy]
      score = h.score < 0 ? 0 : h.score
      @limit[h.key] = (score - score%@step_size)
      @max_limit = @limit[h.key] if @max_limit < @limit[h.key] and @limit[h.key] < @max_size
      @min_limit = @limit[h.key] if @min_limit > @limit[h.key] and @limit[h.key] >= @min_size
      if score >= @limit[h.key]
        h.log[@log_key].file[:values].merge!({ @limit[h.key] => @common_block.length })
        h.previous_score = score
      end
      h.deactivate if h.score <= 0
    end
  end
  
  def teardown
    super
    @index_iterator = @max_limit.step(@min_limit, -@step_size).map { |x| x}
  end
  
  def set_headings
    [as_header("Score")] << @index_iterator[0...-1].map { |x| align(as_header(">= #{x}"))} << align(as_header(0))
  end
  
  def set_title(method)
    as_important_header("Single common block test result (#{$size_converter.number_to_human_size(@filesize)})")
  end
    
  def set_rows(h, log, h_index)
    @rows << [[as_bold(h.class::PROGRAM_NAME)] << log.means.map { |m| ""}].flatten
    @rows << [["Avg. block size"] << align_each(log.means.map{|m| $size_converter.number_to_human_size(m) if m})].flatten
    @rows << [["Avg. block size (%)"] << align_each_with_precision(log.means.map{ |mean| 100*mean/@filesize if mean}, PRECISION[:low])].flatten
    @rows << [["Matches"] << align_each(log.matches)].flatten
    @rows << :separator if (h_index < @hash_functions.length-1)
  end
  
end
