require_relative 'base_hash_function'

class FuzzyHashFunction < BaseHashFunction
  # The matching score of two files.
  attr_accessor :score
  # The log collects the scores for the fragment detection test and the percentual changes of the obfuscation resistance test.
  attr_accessor :log
  # It saves the score computed last. Needed in the Obfuscation resistance test to build the index.
  attr_accessor :previous_score

  # Initialize the attributes.
  def initialize(params="")
    super(params)
    @log = {}
    @all_pair_comparison_command = nil
    @gen_compare_command = nil
    reset
  end
  
  # Reset the basic attributes to default values.
  def reset
    super
    @score = -1
    @previous_score = 100
    @active = true
  end
  
  # Deactivattes a hash function.
  def deactivate
    @active = false
  end
  
  # Determines if a hash function is active.
  def is_active?
    @active
  end

  # Performs an all-against-all-comparison of the digest file. This function is used in the efficiency test to measure the time.
  def all_pairs_comparison
    @duration[:pairs] = realtime {%x[#{@all_pair_comparison_command} #{@digest_file[:name]} > /dev/null]}
  end
  
  # Similar to the all_pairs_comparison, but it does not need a digests file, as it generates tand compares he digests on-the-fly.
  def gen_compare_all(dir)
    %x[#{@gen_compare_command} #{dir}*]
  end
  
  # Computes the score of two files and extracts it from the match string.
  #
  # If the match string is empty, -1 is returned.
  def compute_score(f1, f2)
    @output = %x[#{@gen_compare_command} "#{f1}" "#{f2}"]
    @score = (@output.nil? or @output.empty?) ? -1 : self.class.split_match(@output)[2]
  end

end
